-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio               "EskaQuestTracker.Classes.Scenario"                     ""
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
function OnLoad(self)
  _DB:SetDefault("Scenario", {
    header = {
      name = {
        textColor = { r = 1, g = 0.41, b = 0},
        textSize = 10,
        textFont = "PT Sans Narrow Bold",
        textTransform = "lowercase",
      }
    },
    textSizes = {
      name = 12,
      stageName = 8,
      stageCounter = 10,
    },
    textFonts = {
      name = "PT Sans Narrow Bold",
      stageName = "PT Sans Bold",
      stageCounter = "PT Sans Narrow Bold"
    },
    textTransforms = {
      name = "uppercase",
      stageName = "none",
      stageCounter = "none",
    },
    textColors = {
      name = { r = 1, g = 0.5, b = 0 },
      stageName = { r = 1, g = 1, b = 0 },
      stageCounter = { r = 1, g = 1, b = 1 }
    }
  })
end

class "Scenario" inherit "Block" extend "IObjectiveHolder"
  _ScenarioCache = setmetatable( {}, { __mode = "k" } )

  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  local function SetName(self, new, old, prop)
    self.frame.name:SetText(new)
  end
  local function SetStageName(self, new, old, prop)
    self.frame.stage.name:SetText(new)
  end

  local function SetCurrentStage(self, new, old, prop)
    local txt = self.currentStage .. "/" .. self.numStages
    self.frame.stage.counter:SetText(txt)
  end

  local function SetNumObjectives(self, new, old, prop)
    if new > old then
      for i = 1, new - old do
        local objective = _ObjectManager:GetObjective()
        self:AddObjective(objective)
      end
    elseif new < old then
        for i = 1, old - new do
          local objective = self:GetObjective(new + 1)
          if objective then
            local obj = self.objectives:Remove(objective)
            objective.isReusable = true
          end
        end
      self:Draw()
    end
  end

  -- ======================================================================== --
  -- Methods                                                                  --
  -- ======================================================================== --
  function Refresh(self)

    for element, frame in self.elements:GetIterator() do
      local font = _LibSharedMedia:Fetch("font", Scenario:GetTextFont(element))
      local size = Scenario:GetTextSize(element)
      local color = Scenario:GetTextColor(element)
      local transform = Scenario:GetTextTransform(element)

      local txt = ""
      if element == "name" then txt = self.name
     elseif element == "stageName" then txt = self.stageName
     elseif element == "stageCounter" then txt = self.currentStage .. "/" .. self.numStages end

      if transform == "uppercase" then
        txt = txt:upper()
      elseif transform == "lowercase" then
        txt = txt:lower()
      end
      frame:SetFont(font, size, "OUTLINE")
      frame:SetTextColor(color.r, color.g, color.b)
      frame:SetText(txt)

    end
  end

  __Arguments__{ Objective }
  function AddObjective(self, objective)
    self.objectives:Insert(objective)

    objective:SetParent(self.frame)
    objective:Show()
    self:Draw()
  end

  __Arguments__{ Number }
  function GetObjective(self, index)
    return self.objectives[index]
  end

  __Arguments__{}
  function Draw(self)
    local stage = self.frame.stage
    self:DrawObjectives(stage)
  end

  function Reset(self)
      self.name = nil
      self.currentStage = nil
      self.numStages = nil
      self.stageName = nil
      self.numObjectives = nil
  end

  __Static__() function RefreshAll()
    for obj in pairs(_ScenarioCache) do
      obj:Refresh()
    end
  end


  __Static__() function SetTextSize(self, index, size)
    _DB.Scenario.textSizes[index] = size ; self:RefreshAll()
  end

  __Static__() function GetTextSize(self, index)
    return _DB.Scenario.textSizes[index]
  end

  __Static__() function SetTextFont(self, index, font)
    _DB.Scenario.textFonts[index] = font ; self:RefreshAll()
  end

  __Static__() function GetTextFont(self, index)
    return _DB.Scenario.textFonts[index]
  end

  __Static__() function SetTextColor(self, index, color)
    _DB.Scenario.textColors[index] = color ; self:RefreshAll()
  end

  __Static__() function GetTextColor(self, index)
    return _DB.Scenario.textColors[index]
  end

  __Static__() function SetTextTransform(self, index, transform)
    _DB.Scenario.textTransforms[index] = transform ; self:RefreshAll()
  end

  __Static__() function GetTextTransform(self, index)
    return _DB.Scenario.textTransforms[index]
  end
    --SetDBValue("TextSize", "Name", 10)
    --

  --[[__Static__() function SetTextSize(_, index, value)
    print("GetTextSize", self, index, value)
    _DB.Scenario.
    Scenario:SetConfig("textSize", "Name", 10)
  end--]]

  --GetDBValue("TextSize", "Name")
  -- CreateTextOptions(db, )
  -- Scenario.TextSizes["Name"]

  -- CreateTextOptions()

  -- SetConfig

  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "name" { TYPE = String, DEFAULT = "", HANDLER = SetName}
  property "currentStage" { TYPE = Number, DEFAULT = 1, HANDLER = SetCurrentStage}
  property "numStages"{TYPE = Number, DEFAULT = 1, HANDLER = SetCurrentStage }
  property "stageName" { TYPE = String, DEFAULT = "", HANDLER = SetStageName }
  property "numObjectives" { TYPE = Number, DEFAULT = 0, HANDLER = SetNumObjectives}

  __Static__() property "headerNameTextSize" {
      Set = function(self, size) _DB.Scenario.header.name.textSize = size ; Scenario:RefreshAll() end,
      Get = function(self) return _DB.Scenario.header.name.textSize end
  }
  __Static__() property "headerNameTextFont" {
    Set = function(self, font) _DB.Scenario.header.name.textFont = font ; Scenario:RefreshAll() end,
    Get = function(self) return _DB.Scenario.header.name.textFont end,
  }
  __Static__() property "headerNameTextColor" {
    Set = function(self, color) _DB.Scenario.header.name.textColor = color ; Scenario:RefreshAll() end,
    Get = function(self) return _DB.Scenario.header.name.textColor end,
  }
  __Static__() property "headerNameTextTransform" {
    Set = function(self, transform) _DB.Scenario.header.name.textTransform = transform ; Scenario:RefreshAll() end,
    Get = function(self) return _DB.Scenario.header.name.textTransform end,
  }
  -- stage
  --property "textSizes" {
  --  Set = function(self, size)
  --}
  __Static__() property "textSizes" {
    Set = function(self, ...)
    end,
    Get = function() return _DB.Scenario.textSizes end
  }




function Scenario(self)
  Super(self, "Scenario", 10)
  self.text = "Scenario"

  local header = self.frame.header
  local headerText = header.text

  self.frame:SetBackdropColor(0, 0, 0, 0.3)
  self.frame:SetBackdropBorderColor(0, 0, 0, 0)

  -- Scenario name
  local name = header:CreateFontString(nil, "OVERLAY")
  --name:SetWidth(50)
  --name:SetFont(_Addon.Fonts.Test, 12)
  --name:SetText(self.name)
  name:SetPoint("TOPRIGHT")
  name:SetPoint("BOTTOMRIGHT")
  name:SetPoint("LEFT", headerText, "RIGHT")
  name:SetJustifyH("CENTER")
  self.frame.name = name

  -- Set the headerText to left
  headerText:SetPoint("LEFT")
  headerText:SetJustifyH("LEFT")
  headerText:SetWidth(75)
  -- Stage Number
  --local stageName



  -- Stage Frame
  local stage = CreateFrame("Frame", nil, self.frame)
  stage:SetPoint("TOPLEFT", header, "BOTTOMLEFT")
  stage:SetPoint("TOPRIGHT", header, "BOTTOMRIGHT")
  stage:SetBackdrop(_Backdrops.Common)
  stage:SetBackdropBorderColor(0, 0, 0, 0.4)
  stage:SetBackdropColor(0, 0, 0, 0.4)
  stage:SetHeight(15)
  self.frame.stage = stage

  -- Stage Counter
  local stageCounter = stage:CreateFontString(nil, "OVERLAY")
  stageCounter:SetPoint("TOPLEFT")
  stageCounter:SetPoint("BOTTOMLEFT")
  --stageCounter:SetFont(_Addon.Fonts.Test, 12)
  --stageCounter:SetText(self.stageCounter)
  stageCounter:SetWidth(50)
  stage.counter = stageCounter

  -- Stage name
  local stageName = stage:CreateFontString(nil, "OVERLAY")
  stageName:SetPoint("TOPRIGHT")
  stageName:SetPoint("TOPLEFT", stageCounter, "TOPRIGHT")
  stageName:SetPoint("BOTTOMRIGHT")
  stageName:SetPoint("BOTTOMLEFT", stageCounter, "BOTTOMRIGHT")
  --stageName:SetFont(_Addon.Fonts.Test, 12)
  --stageName:SetText(self.stageName)
  stage.name = stageName


  self.height = self.height + stage:GetHeight()
  self.baseHeight = self.height


  -- Register the differents elements
  self.elements = Dictionary()
  self.elements["name"] = name
  self.elements["stageCounter"] = stageCounter
  self.elements["stageName"] = stageName
  self:Refresh()

  self.objectives = ObjectArray(Objective)

  _ScenarioCache[self] = true
end


endclass "Scenario"
